@@ -8,7 +8,6 @@ from . import card_info
 
 
 class Target(database.KeyStoredObject, abc.ABC):
-    SUPPORTED_OS = ["posix", "nt"]
     SUPPORTS_RELNK = False
 
     def __init__(self, name, short, file_extension, needs_card_info=True):
@@ -29,10 +28,10 @@ class Target(database.KeyStoredObject, abc.ABC):
             return utils.expand_path(os.path.join("~", "Decks"))
 
     @abc.abstractmethod
-    def save_deck(self, deck, path, card_info_map=None):
+    def save_deck(self, deck, path, include_maybe=False, card_info_map=None):
         """Writes deck to path in format using card_info_map."""
 
-    def save_decks(self, deck_tuples, card_info_map=None):
+    def save_decks(self, deck_tuples, include_maybe=False, card_info_map=None):
         if card_info_map is None:
             card_info_map = card_info.map_from_decks(
                 [d for d, _ in deck_tuples],
@@ -40,7 +39,7 @@ class Target(database.KeyStoredObject, abc.ABC):
             )
 
         for deck, path in deck_tuples:
-            self.save_deck(deck, path, card_info_map)
+            self.save_deck(deck, path, include_maybe, card_info_map)
 
     def create_file_name(self, deck_name):
         return utils.create_file_name(deck_name) + self.file_extension
